#!/usr/bin/env nextflow

nextflow.enable.dsl=2
include { LIPMUTILS_download as DOWNLOAD_dfam; LIPMUTILS_download as DOWNLOAD_transposonPsi } from './modules/software/builddb/main.nf'
include { EGNPCORE_dfam2fasta;LIPMUTILS_fixIds } from './modules/software/builddb/main.nf'
include { LIPMUTILS_idxRepeat as IDX_dfam; LIPMUTILS_idxRepeat as IDX_transposonpsi } from './modules/software/builddb/main.nf'

include { LIPMUTILS_download as DOWNLOAD_sprot } from './modules/software/builddb/main.nf'
include { EGNPCORE_uniprot2fasta as CONVERT_sprot } from './modules/software/builddb/main.nf'
include { LIPMUTILS_splitSingleDB as SPLIT_sprot } from './modules/software/lipmutils/main.nf'
include { USEARCH_blastRepeat as BLASTREPEAT_sprot } from './modules/software/builddb/main.nf'
include { LIPMUTILS_filterBlast as FILTERBLASTREPEAT_sprot } from './modules/software/builddb/main.nf'
include { LIPMUTILS_filterProtDB as FILTERFASTA_sprot } from './modules/software/builddb/main.nf'

include { LIPMUTILS_download as DOWNLOAD_trembldomain } from './modules/software/builddb/main.nf'
include { EGNPCORE_uniprot2fasta as CONVERT_trembldomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_splitSingleDB as SPLIT_trembldomain } from './modules/software/lipmutils/main.nf'
include { USEARCH_blastRepeat as BLASTREPEAT_trembldomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_filterBlast as FILTERBLASTREPEAT_trembldomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_filterProtDB as FILTERFASTA_trembldomain } from './modules/software/builddb/main.nf'
include { CDHIT_compress as COMPRESS_trembldomain } from './modules/software/builddb/main.nf'

include { LIPMUTILS_download as DOWNLOAD_refdomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_splitSingleDB as SPLIT_refdomain } from './modules/software/lipmutils/main.nf'
include { USEARCH_blastRepeat as BLASTREPEAT_refdomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_filterBlast as FILTERBLASTREPEAT_refdomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_filterProtDB as FILTERFASTA_refdomain } from './modules/software/builddb/main.nf'
include { LIPMUTILS_getUniprotRelease } from './modules/software/builddb/main.nf'



workflow {
	main:

		if (params.help) {
			def help_message = NFcoreTemplate.help(workflow, params, log)
			log.info help_message
			exit 0
		}

		if(! params.outdir) { exit 1, 'outdir not specified' }
		if(! params.store) { exit 1, 'store not specified' }
		if(! params.domain) { exit 1, 'domain not specified' }
		if(! params.dfam_version) { exit 1, 'dfam_version not specified' }

		currentdate = Utils.getCurrentDate();
		if (params.date) {
			currentdate = params.date
		}
		
		
		
		if (! params[params.domain] ) {
			Utils.error("domain ${params.domain} unknown (plants, fungi, nematodes, oomycetes, arthropods)" )
		}
		
		trembldomain_url = params[params.domain].trembldomain_url
		refproteome_url = params[params.domain].refproteome_url
		refproteome_name = params[params.domain].refproteome_name
				

		dfam_dat = DOWNLOAD_dfam("${params.dfam_url}/Dfam_${params.dfam_version}/families/Dfam.embl.gz",'Dfam',params.dfam_version, 'embl.gz')
		transposon_raw = DOWNLOAD_transposonPsi(params.transposonpsi_url,'transposon_db','pep','raw')
		
		dfam_fasta = EGNPCORE_dfam2fasta(dfam_dat)
		transposon_fasta = LIPMUTILS_fixIds(transposon_raw,'transposon_db','pep','raw')
		
		dfam_idx = IDX_dfam(dfam_fasta,'nucl')
		transposonpsi_idx = IDX_transposonpsi(transposon_fasta,'prot')

		
		uniprot_release = LIPMUTILS_getUniprotRelease()
		
		sprot_name = "uniprot_sprot"
		
		
		sprot_dat = DOWNLOAD_sprot(params.sprot_url,"${sprot_name}", uniprot_release, "dat.gz")
		
		sprot_fasta = CONVERT_sprot(sprot_dat, 10, 50)
		sprot_slice = SPLIT_sprot(sprot_name, sprot_fasta, 1000).flatten()
		sprot_m8 = BLASTREPEAT_sprot(sprot_slice,dfam_fasta,dfam_idx,transposon_fasta,transposonpsi_idx,params.blast_params)
		sprot_m8_filter = FILTERBLASTREPEAT_sprot(sprot_m8.collect(), sprot_name)
		sprot_noTE = FILTERFASTA_sprot(sprot_name,sprot_fasta,sprot_m8_filter)
		
		trembldomain_name = "uniprot_trembl_${params.domain}"
		trembldomain_dat = DOWNLOAD_trembldomain(trembldomain_url,"${trembldomain_name}",uniprot_release, "dat.gz")
		trembldomain_fasta = CONVERT_trembldomain(trembldomain_dat, 3, 50)
		trembldomain_slice = SPLIT_trembldomain(trembldomain_name, trembldomain_fasta, 1000).flatten()
		trembldomain_m8 = BLASTREPEAT_trembldomain(trembldomain_slice,dfam_fasta,dfam_idx,transposon_fasta,transposonpsi_idx,params.blast_params)
		trembldomain_m8_filter = FILTERBLASTREPEAT_trembldomain(trembldomain_m8.collect(), trembldomain_name)
		trembldomain_noTE = FILTERFASTA_trembldomain(trembldomain_name,trembldomain_fasta,trembldomain_m8_filter)
		trembldomain_final = COMPRESS_trembldomain(trembldomain_name,trembldomain_noTE,1024000)
		
		
		//refproteome_name refproteome_url
		refdomain_name = "refproteome.${params.domain}"
		refdomain_fasta = DOWNLOAD_refdomain(refproteome_url,"${refdomain_name}", currentdate, 'fasta.gz')
		refdomain_slice = SPLIT_refdomain(refdomain_name, refdomain_fasta, 1000).flatten()
		refdomain_m8 = BLASTREPEAT_refdomain(refdomain_slice,dfam_fasta,dfam_idx,transposon_fasta,transposonpsi_idx,params.blast_params)
		refdomain_m8_filter = FILTERBLASTREPEAT_refdomain(refdomain_m8.collect(), refdomain_name)
		refdomain_noTE = FILTERFASTA_refdomain(refdomain_name,refdomain_fasta,refdomain_m8_filter)
		
		
		publish(refdomain_noTE,sprot_noTE,trembldomain_final,dfam_fasta,transposon_fasta ,params.domain)
		dbinfo(currentdate,uniprot_release, trembldomain_url,params.sprot_url,params.blast_params,refproteome_url,refproteome_name)			
	
		
		
		workflow.onComplete {
    		Utils.info("Pipeline completed", true)
    	}
		workflow.onError {
    		Utils.error("Oops... Pipeline execution stopped") //": ${workflow.errorMessage}"
		}
}

process publish {
	publishDir("${params.outdir}/", mode:'copy')
	
	input:
	path(ref)
	path(sprot)
	path(trembl)
	path(dfam)
	path(transposonpsi)
	val(domain)
	
	output:
	path("*.fasta.gz")
	
	script:
	"""
	gzip -c ${ref} > refproteome.${params.domain}.fasta.gz
	gzip -c ${sprot} > uniprot_sprot_cleaned.fasta.gz
	gzip -c ${trembl} > uniprot_trembl_cleaned.${params.domain}.fasta.gz
	gzip -c ${dfam} > Dfam.fasta.gz
	gzip -c ${transposonpsi} > transposonpsi.fasta.gz
	"""
}


process dbinfo {
	publishDir("${params.outdir}/", mode:'copy')
	
	input:
	val(date)
	val(uniprot_release)
	val(trembl_url)
	val(sprot_url)
	val(blast_params)
	val(ref_url)
	val(ref_name)
	
	output:
	path("${params.domain}.info")
	
	script:
	"""
	echo "domain='${params.domain}'" > ${params.domain}.info
	echo "date='${date}'"  >> ${params.domain}.info
	echo "uniprot_release='${uniprot_release}'"  >> ${params.domain}.info
	echo "trembl_url='${trembl_url}'" >> ${params.domain}.info
	echo "sprot_url='${sprot_url}'" >> ${params.domain}.info
	echo "reference_url='${ref_url}'" >> ${params.domain}.info
	echo "reference_name='${ref_name}'" >> ${params.domain}.info
	echo "dfam_version='${params.dfam_version}'" >> ${params.domain}.info
	echo "cdhit_params='${params.cdhit_params}'" >> ${params.domain}.info
	echo "blast_params='${blast_params}'" >> ${params.domain}.info
	"""
}


