process HELIXER_getModels {
    
    output:
    stdout
    
    script:
    """
    HOME=\$PWD && fetch_helixer_models.py > fetch.out
    echo \$HOME | tr -d '\\n'
    """
}


process HELIXER_run {

    label "process_high_memory"
    tag("${sequence.baseName}")
    
    input:
    path(sequence)
    val(modelPath)

    output:
    tuple val(sequence.baseName), path(sequence), path("${sequence.baseName}.${source}.gff3")
    
    script:
    
    source = 'helixer'
    lineage = 'land_plant'

    if (params.domain == 'fungi' || params.domain == 'oomycetes') {
        lineage = 'fungi'
    } else if (params.domain == 'nematodes' ||  params.domain == 'arthropods') {
        lineage = 'invertebrate'
    }


    """
    HOME=${modelPath} && Helixer.py --no-multiprocess --temporary-dir \$PWD --lineage ${lineage} --gff-output-path ${sequence.baseName}.${source}.gff3 --fasta-path ${sequence} 
    [ -s ${sequence.baseName}.${source}.gff3 ] && touch ${sequence.baseName}.${source}.gff3 || echo "##gff-version 3" > ${sequence.baseName}.${source}.gff3
    """
}

process HELIXER_softwareInfo {
	output:
	path("helixer.software_info.tsv")

	script:
	"""
    echo -e "#software\tversion\tsource\tlicense\tcitation" > helixer.software_info.tsv
	echo -e "helixer\t${params.version.helixer}\thttps://github.com/gglyptodon/helixer-docker\tGPLv3\tFelix Stiehler, Marvin Steinborn, Stephan Scholz, Daniela Dey, Andreas P M Weber, Alisandra K Denton, Helixer: cross-species gene annotation of large eukaryotic genomes using deep learning, Bioinformatics, Volume 36, Issue 22-23, December 2020, Pages 5291–5298" >> helixer.software_info.tsv
	"""
}
