nextflow.enable.dsl=2


process LIPMUTILS_createOutputDirectory {
    
    publishDir "${params.outdir}/99_workdir", mode: 'copy', enabled: params.publish_workdir
    tag("Split ${sequences}")
    

    input:
    path(sequences)

    output:
    path('entries.fof.sequences')
    path('sequences')

    script:
    """
    lipm_fasta2tree.pl --sequence ${sequences}  --outdir \$PWD
    perl -ne 's,/\\d{4}/(\\S+/\\S+)\$,/sequences/\$1,;print' entries.fof  > entries.fof.sequences
    mkdir -p sequences
    cd sequences && mv ../0*/* .
    """
}

process LIPMUTILS_trimPolyX {
    
    tag("Trim ${estConf.name}")

    input:
    val(estConf)
    
    output:
    tuple val(estConf.name), path("clean/${outfile}")
    path ("${estConf.name}.conf")
    
    script:
    outfile = estConf.file.split('/').last()

    trim_cmd = "cd clean && ln -s ${estConf.file} .  && cd .."
    if (estConf.trim == 1)
    {
        trim_cmd = "lipm_trim_polyX.pl --in ${estConf.file} --out clean/${outfile}"
    }
    """
    mkdir -p clean
    ${trim_cmd}
    file=`realpath clean/${outfile}`
    entries=`grep -c "^>" clean/${outfile}`
    echo "#${estConf.id}" > ${estConf.name}.conf
    echo "est_${estConf.id}_pcs=${estConf.pcs}" >> ${estConf.name}.conf
    echo "est_${estConf.id}_pci=${estConf.pci}" >> ${estConf.name}.conf
    echo "est_${estConf.id}_preserve=${estConf.preserve}" >> ${estConf.name}.conf
    echo "est_${estConf.id}_training=${estConf.training}" >> ${estConf.name}.conf
    echo "est_${estConf.id}_priority=${estConf.priority}" >> ${estConf.name}.conf
    echo "est_${estConf.id}_remove_unspliced=${estConf.remove_unspliced}" >> ${estConf.name}.conf
    echo "est_${estConf.id}_file=\$file" >> ${estConf.name}.conf
    echo "est_${estConf.id}_entries=\$entries" >> ${estConf.name}.conf
    """
}

process LIPMUTILS_fasta2tree {
    tag("Split ${sequences}")
    
    input:
    path(sequences)

    output:
    path('entries.fof.sequences')
    path('sequences')

    script:
    """
    lipm_fasta2tree.pl --sequence ${sequences}  --outdir \$PWD
    perl -ne 's,/\\d{4}/(\\S+/\\S+)\$,/sequences/\$1,;print'  entries.fof  > entries.fof.sequences
    mkdir -p sequences
    cd sequences && mv ../0*/* .
    """
}

process LIPMUTILS_fasta2overlappingwins {
    
    tag("Get overlapping windows ${sequences}")

    input:
    path(sequences)
    val(largeWindowFactor)

    output:
    path('overlapping.fasta')

    script:
    maxlen = params.windowMaxLen * largeWindowFactor
    step =  maxlen - params.windowOverlapLen
    """
    lipm_fasta2overlappingwins.pl --in ${sequences}  --out \$PWD/overlapping.fasta --max ${maxlen} --step ${step}
    """
}

process LIPMUTILS_splitBed {
 
    tag("${infile.baseName}")

	input:
    path(infile)
    path(inputSequences)

	output:
    path("*.${infile.baseName}.bed") 
    
    script:
    """
    grep '^>' ${inputSequences} | cut -d ' ' -f1 | sed 's/>//' | xargs -i touch {}.${infile.baseName}.bed
    lipm_bed_split_by_sequence.pl --in ${infile}
    """

}
process LIPMUTILS_bedToGff3 {

    tag("${seqid}")
	
	input:
    val(source)
    val(feature_type)
    tuple val (seqid), path(infile)

    output:
    tuple val (seqid), path("${seqid}.${source}.gff3")

    script:
    if( source == 'red' )
    """
    [ -s ${infile} ] && lipm_bed_to_gff3.pl --in ${infile} --out ${seqid}.${source}.gff3.raw  --source ${source} --code ${feature_type} ||  echo "##gff-version 3" > ${seqid}.${source}.gff3.raw
    perl -ne 'chomp;@a = split("\\t");if (\$a[6] eq ".") {\$a[6] = "+";} print join("\\t",@a) . "\\n"' ${seqid}.${source}.gff3.raw > ${seqid}.${source}.gff3
    """
    else
    """
    [ -s ${infile} ] && lipm_bed_to_gff3.pl --in ${infile} --out ${seqid}.${source}.gff3  --source ${source} --code ${feature_type} ||  echo "##gff-version 3" > ${seqid}.${source}.gff3
    """
    
}

process LIPMUTILS_m8ToGff3 {
	
    tag("${seqid}")

    input:
    tuple val(proteinConf), val(seqid), path(m8file)

    output:
    tuple val (proteinConf), val(seqid), path("${seqid}.${dbname}.raw.gff3")

    script:
    dbname = proteinConf.name
    """
    lipm_m8_to_gff3.pl --infile ${m8file} --source ${dbname} --outfile ${seqid}.${dbname}.raw.gff3
    [ -s ${seqid}.${dbname}.raw.gff3 ] && touch ${seqid}.${dbname}.raw.gff3 ||  echo "##gff-version 3" > ${seqid}.${dbname}.raw.gff3
    """
}

process LIPMUTILS_m8RepeatToGff3 {
	
    tag("${seqid}")

    input:
    tuple val(seqid), val(dbname), path(m8file)

    output:
    tuple val(seqid), path("${seqid}.masked.${dbname}.gff3")

    script:
    
    """
    lipm_m8_to_gff3.pl --infile ${m8file} --source ${dbname} --outfile ${seqid}.masked.${dbname}.gff3
    [ -s ${seqid}.masked.${dbname}.gff3 ] && touch ${seqid}.masked.${dbname}.gff3 ||  echo "##gff-version 3" > ${seqid}.masked.${dbname}.gff3
    """
}


process LIPMUTILS_splitEstDBs {
    
    label "process_high_memory"
	tag("${conf.name}")
    
    input:
    val(conf)
    
    output:
    path("${conf.name}.*")
    
    script:
    """
    lipm_fastasplitter.pl --in ${conf.file} --num_per_slice ${params.est_num_per_slice} --outprefix ${conf.name}.
    """
}

process LIPMUTILS_splitSingleDB {
    
    label "process_high_memory"

    tag("${name}")

    input:
    val(name)
    path(file)
    val(num_per_slice)
    
    output:
    path("${name}.*")
    
    script:
    """
    lipm_fastasplitter.pl --in ${file} --num_per_slice ${num_per_slice} --outprefix ${name}.
    """
}

process LIPMUTILS_fastaFilterMinLen {

    tag("${infile.baseName}")

    label "process_high_memory"

    input:
    path(infile)
    val(minlen)

       output:
       path("${infile.baseName}.filter.fasta")
    
       script:
       """
       lipm_fastafilter.pl --in ${infile} --out ${infile.baseName}.filter.fasta --min_length ${minlen}
       """
}

process LIPMUTILS_fastaFilterIdsAndMinLenMulti {
    tag("${infile}")

    input:
    path(ids)
    val(minlen)
    path(infile)
    
	output:
	path("${infile}.filter.fasta")
    
	script:
	"""
    [ -s ${ids} ] && lipm_fastafilter.pl --in ${infile} --out ${infile}.filter.fasta --min_length ${minlen} --db_filename ${ids} --db_col 1 --keep || touch ${infile}.filter.fasta 
	"""
}


process LIPMUTILS_nrRename {

    tag("${infile.baseName}")
    
    input:
    path(infile)
    

	output:
	path("${infile.baseName}.clean.fasta")
    
	script:
	"""
	cut -d ' ' -f1 ${infile} > ${infile}.tmp
	lipm_nrdb -o ${infile}.tmp.nr ${infile}.tmp
	lipm_nr_rename.pl --in ${infile}.tmp.nr --out ${infile.baseName}.clean.fasta
	rm -f ${infile}.tmp*
	"""
}

process LIPMUTILS_annotationStatistics {

	publishDir "${params.outdir}/04_statistics/", mode: 'copy'

	input:
	tuple val(seqid), path(sequence), path(gff3)
	
	output:
	path("${params.output_prefix}.????????.*")
	
	script:
	"""
	lipm_genome_statistics.pl --in ${gff3} --scaffolds ${sequence} --kingdom ${params.kingdom} --out ${params.output_prefix}.`date '+%Y%m%d'`
	"""
}


process LIPMUTILS_transfertGff3Attributes {

	tag("${seqid}")
	errorStrategy { task.exitStatus in 0..1 ? 'ignore' : 'terminate' }

	input:
	tuple val(seqid), path(annotGff3), path(ncrnaGff3)
	
	output:
	tuple val(seqid), path("${seqid}.clean.gff3")
	
	script:
	"""
	echo '###' > remove.ids
    [ -s ${seqid}.trnascan.gff3 ] && touch ${seqid}.trnascan.gff3 || (echo "##gff-version 3" && cat  ${seqid}.trnascan.*.gff3 | grep -v '^#' || true) > ${seqid}.trnascan.gff3
    [ -s ${seqid}.barrnap.gff3 ] && touch ${seqid}.barrnap.gff3 || (echo "##gff-version 3" && cat ${seqid}.barrnap.*.gff3 | grep -v '^#' || true) > ${seqid}.barrnap.gff3
    [ -s ${seqid}.rfamscan.gff3 ] && touch ${seqid}.rfamscan.gff3 || (echo "##gff-version 3" && cat ${seqid}.rfamscan.*.gff3  | grep -v '^#' || true) > ${seqid}.rfamscan.gff3
    grep -v ^### ${seqid}.rfamscan.gff3 > ${seqid}.rfamscan.raw.gff3 
    grep -v ^### ${seqid}.trnascan.gff3 > ${seqid}.trnascan.raw.gff3 
    grep -v ^### ${seqid}.barrnap.gff3 > ${seqid}.barrnap.raw.gff3 
    bedtools intersect -s -a ${seqid}.rfamscan.raw.gff3 -b ${seqid}.trnascan.raw.gff3 |cut -f9 | cut -d ';' -f1 >> remove.ids
    bedtools intersect -s -a ${seqid}.rfamscan.raw.gff3 -b ${seqid}.barrnap.raw.gff3 |cut -f9 | cut -d ';' -f1 >> remove.ids
    ( grep -w -v -f remove.ids  ${seqid}.rfamscan.raw.gff3 || true ) > ${seqid}.rfamscan.clean.gff3
    ( echo "##gff-version 3" && (cat ${seqid}.rfamscan.clean.gff3  ${seqid}.trnascan.gff3 ${seqid}.barrnap.gff3 | grep -v ^#) || true) > ${seqid}.allncrna.gff3
    lipm_transfer_gff3_attributes.pl --in ${annotGff3} --ref ${seqid}.allncrna.gff3  --out ${seqid}.clean.gff3 --intersectbed_type "tRNA;rRNA;ncRNA" --attrs "subunit;rfam_acc;rfam_id;anti_codon;product"
	"""
}


process LIPMUTILS_splitProtDBs {
    
    tag("${conf.name}")
    label "process_high_memory"

    input:
    tuple val(conf),path(file)
    val(num_per_slice)
    
    output:
    path("${conf.name}.*")
    
    script:
    """
    lipm_fastasplitter.pl --in ${file} --num_per_slice ${num_per_slice} --outprefix ${conf.name}.
    """
}


process LIPMUTILS_sortGff3 {
    
    tag("${prefix}")
    label "process_high_memory"
    
    input:
    tuple val(prefix),path(gff3)

    output:
    tuple val(prefix),path("${prefix}.sort.gff3")

    script:
    """
    lipme_fastSortGff3.pl --mem ${task.memory} --input ${gff3}  --output  ${prefix}.sort.gff3 
    """
}


process LIPMUTILS_extractCDSFromHelixer {

	tag("${seqid}")
	
    input:
    tuple val(seqid), path(sequence), path("helixerGff3")
    
    output:
    path("${seqid}.helixer.cds.fasta")
    
    script:

    min_length = params.hsp_training_length * 3

	// TODO EXTRAIRE CDS AVEC BFC  BIOFILECONVERTER_gff2fasta
    """
    grep -P "\\tCDS\\t" ${helixerGff3} | grep -P '\\t\\-\\t' |  cut -f1,4,5 | sed 's/\\t/:/;s/\\t/-/' > cds.rev
    grep -P "\\tCDS\\t" ${helixerGff3} | grep -P '\\t\\+\\t' |  cut -f1,4,5 | sed 's/\\t/:/;s/\\t/-/' > cds.fw
    samtools faidx  ${sequence} --reverse-complement --mark-strand sign --region-file cds.rev > cds.fasta
    samtools faidx  ${sequence} --mark-strand sign --region-file cds.fw >> cds.fasta
    lipm_fastafilter.pl --in cds.fasta --min_length ${min_length} --out ${seqid}.helixer.cds.fasta
    """
}

process LIPMUTILS_HelixerCDS4Training {
    
    input:
    path(helixerCds)
    path(skipLowComplexity)

    output:
    path("helixer.cds.tsv")
    
    
    script:
    """
    cat ${skipLowComplexity} > cds.skip.txt
    cat ${helixerCds} > cds.helixer.fasta.tmp
    lipm_fastafilter.pl --in  cds.helixer.fasta.tmp --out  cds.helixer.fasta --remove --db_column 1  --db_filename cds.skip.txt
    perl -ne 'chomp; (\$b) = (/begin=(\\d+)/); (\$e) = (/end=(\\d+)/); (\$strand) = (/strand=(\\S+)/); \$s = "+"; if (\$strand <0 ) {\$s = "-"};s/>(\\S+).+/\\n\$1-\$b,\$e,\$s, /;s/(T[GA]A|TAG)\$//;print'  cds.helixer.fasta | grep ',' > helixer.cds.tsv
    echo "" >>  helixer.cds.tsv
    """
}

process LIPMUTILS_HelixerGff2match {

	tag("${seqid}")
	label "process_high"
	
	input:
	tuple val(seqid), path(sequence), path(gff)
	path(proteins)
	
	output:
	tuple val(seqid),  path("${seqid}.blast0.gff3")
	path("${seqid}.skip.txt")
	
	script:
	"""
	touch ${seqid}.skip.txt
	[ -s ${seqid}_prot.fasta  ] && lipme_helixerGff2match.pl --max_lowcomp_perc=90 --gff=${gff} --genome=${sequence} --proteins=${seqid}_prot.fasta --out ${seqid}.blast0.gff3 --cpus=${task.cpus} > ${seqid}.skip.txt || echo "##gff-version 3" > ${seqid}.blast0.gff3
	"""
}



process LIPMUTILS_softwareInfo {
	output:
	path("lipmutils.software_info.tsv")

	script:
	"""
	software_info > lipmutils.software_info.tsv
	"""
}

