nextflow.enable.dsl=2

process LIPMUTILS_validAlignment {

	tag("${estName}")

    input:
    tuple val(estName), path(toFilterFile), path(forValidationFile)
    val(parent)
    val(child)

    output:
	tuple val(estName), path("*.valid.gff3")

	script:
	"""
	outprefix=`basename ${toFilterFile} | sed -E 's/.gff.+//'`
	lipm_validate_spliced_alignements.pl --gff-to-filter ${toFilterFile} --gff-for-validation ${forValidationFile} ${params.alignment_validation_param} --outprefix \$outprefix --parent ${parent} --child ${child} --verbose 2> \$outprefix.validation.report
	"""
}

process MINIMAP2_map {
    label "process_high"
    tag("${estName}")
    
    input:
    path(completeGenome)
    tuple val(estName), val(estConf), path(estSplitFile)
    
    
    output:
    tuple val(estName), path("${outfile}.minimap2_filtered.gff3"), val(estSplitFile)
    tuple val(estName), path("${outfileRmSpliced}.minimap2_filtered.gff3"), val(estSplitFile)  optional true
    tuple val(estName), path("${outfile}.minimap2_raw.gff3.gz"), val (estSplitFile)

    script:
    
    
    outfile = "${completeGenome.baseName}.${estSplitFile}"
    outfileRmSpliced = "${completeGenome.baseName}.${estSplitFile}.rm_unspliced_${params.minimap2_min_len_short_unspliced}"

	
    if (estConf.remove_unspliced as Integer > 0)
    
    """
    minimap2 -c -K 2G -t ${task.cpus} ${params.minimap2_param} ${completeGenome} ${estSplitFile} > ${estSplitFile}.paf
    lipm_minimap2_paf2gff.pl  --outfmt gene --paffile ${estSplitFile}.paf --outfile ${outfile}.minimap2_raw.gff3.gz --trim-end-exon ${params.minimap2_smallexons_minlen}
    egn_minimap_filter.pl --in ${outfile}.minimap2_raw.gff3.gz --out ${outfile}.gff3 --format est --pci ${estConf.pci} --pcs ${estConf.pcs} --min_length_short_unspliced ${params.minimap2_min_len_short_unspliced}
    [ -s ${outfile}.gff3 ] && minimap_remove_aln.pl --in ${outfile}.gff3 --out ${outfile}.minimap2_filtered.gff3  --min_exon_len=${params.minimap2_filter_min_exon_len} --max_short_exon_number=${params.minimap2_filter_max_short_exon_number}  || echo '##gff-version 3' >  ${outfile}.filtered.gff3
    [ -s ${outfileRmSpliced}.gff3 ] && minimap_remove_aln.pl --in ${outfileRmSpliced}.gff3 --out ${outfileRmSpliced}.minimap2_filtered.gff3  --min_exon_len=${params.minimap2_filter_min_exon_len} --max_short_exon_number=${params.minimap2_filter_max_short_exon_number}  || echo '##gff-version 3' >  ${outfileRmSpliced}.filtered.gff3
    """
    
    else
    
    """
    minimap2 -c -K 2G -t ${task.cpus} ${params.minimap2_param} ${completeGenome} ${estSplitFile} > ${estSplitFile}.paf
    lipm_minimap2_paf2gff.pl  --outfmt gene --paffile ${estSplitFile}.paf --outfile ${outfile}.minimap2_raw.gff3.gz --trim-end-exon ${params.minimap2_smallexons_minlen}
    egn_minimap_filter.pl --in ${outfile}.minimap2_raw.gff3.gz --out ${outfile}.gff3 --format est --pci ${estConf.pci} --pcs ${estConf.pcs} --min_length_short_unspliced ${params.minimap2_min_len_short_unspliced}
    [ -s ${outfile}.gff3 ] && minimap_remove_aln.pl --in ${outfile}.gff3 --out ${outfile}.minimap2_filtered.gff3  --min_exon_len=${params.minimap2_filter_min_exon_len} --max_short_exon_number=${params.minimap2_filter_max_short_exon_number}  || echo '##gff-version 3' >  ${outfile}.filtered.gff3
    """
}


process MINIMAP2_softwareInfo {
	output:
	path("minimap2.software_info.tsv")

	script:
	"""
	software_info > minimap2.software_info.tsv
	"""
}

